#ifndef _SB_H_
#define _SB_H_
#include <stdio.h>
#include <dos.h>

unsigned int BASE;
int ready;

int INIT_SB();
unsigned int RESET_DSP(unsigned);
void WRITE_DSP(char);
char READ_DSP();
void WRITE_DAC(char);
char READ_DAC();
void SPKR_ON();
void SPKR_OFF();
void DMA_STOP();
void DMA_CONTINUE();
void interrupt SB_HANDLER(...);
void PLAY(void far *, unsigned, unsigned, unsigned);

int INIT_SB()
{
  unsigned int TEMP = 0x210;

  while (TEMP <= 0x280)
  {
    BASE |= RESET_DSP(TEMP);
    TEMP += 0x10;
  }

  return BASE;
}

void interrupt SB_HANDLER(...)
{
  inportb(BASE + 0x0E);
  ready = 1;
  outportb(0x20, 0x20);
}

unsigned int RESET_DSP(unsigned port_)
{
  outportb(port_ + 6, 1);
  delay(10);
  outportb(port_ + 6, 0);
  delay(10);
  if (((inportb(port_ + 0x0E) & 0x80) == 0x80) &
       (inportb(port_ + 0x0A) == 0xAA))
    return port_;

  return 0;
}

void WRITE_DSP(char value)
{
  while (inportb(BASE + 0x0C) & 0x80) {}
  outportb(BASE + 0x0C, value);
}

char READ_DSP()
{
  while (!(inportb(BASE + 0x0E) & 0x80)) {}
  return inportb(BASE + 0x0A);
}

void WRITE_DAC(char level)
{
  WRITE_DSP(0x10);
  WRITE_DSP(level);
}

char READ_DAC()
{
  WRITE_DSP(0x20);
  return READ_DSP();
}

void SPKR_ON()
{
  WRITE_DSP(0xD1);
}

void SPKR_OFF()
{
  WRITE_DSP(0xD3);
}

void DMA_STOP()
{
  WRITE_DSP(0xD0);
}

void DMA_CONTINUE()
{
  WRITE_DSP(0xD4);
}

void PLAY(void far *sound, unsigned size, unsigned freq, unsigned DMA)
{
  unsigned time_const;
  unsigned page, offs;
  unsigned DMA_Mask = DMA | 4;

  ready = 0;
  size--;

  offs = (FP_SEG(sound) << 4) + FP_OFF(sound);
  page = (FP_SEG(sound) + (FP_OFF(sound) >> 4)) >> 12;

  outportb(0x0A, DMA_Mask);
  outportb(0x0C, 0);
  outportb(0x0B, 0x48 | DMA);
  outportb(0x02, offs & 255);
  outportb(0x02, offs >> 8);
  outportb(0x83, page);
  outportb(0x03, size & 255);
  outportb(0x03, size >> 8);
  outportb(0x0A, DMA);

  time_const = 256 - (1000000 / freq);
  WRITE_DSP(0x40);
  WRITE_DSP(time_const);
  WRITE_DSP(0x14);
  WRITE_DSP(size & 255);
  WRITE_DSP(size >> 8);
}

#endif
